import React from "react"
import { useRef, useEffect, useState } from "react"
import { useTheme } from "../hooks/useTheme"
import { tokenize } from "../utils/tokenizer"
import type { HighlighterProps, Token } from "../types"
import { ChevronDown, ChevronUp } from "lucide-react"

export const SyntaxHighlighter: React.FC<HighlighterProps> = ({
  code,
  language,
  showLineNumbers = true,
  wrapLines = false,
  maxHeight,
  isDarkMode,
  expandable = false,
}) => {
  const { theme } = useTheme()
  const codeRef = useRef<HTMLPreElement>(null)
  const [isExpanded, setIsExpanded] = useState(!expandable)

  if (!code) {
    return <div>No code to display</div>
  }

  useEffect(() => {
    if (codeRef.current) {
      codeRef.current.innerHTML = ""
      const tokens = tokenize(code, language)
      if (tokens.length === 0) return

      let lineNumber = 1
      let lineContent = ""

      tokens.forEach((token: Token) => {
        const span = document.createElement("span")
        span.textContent = token.content
        span.className = `token ${token.type}`
        span.style.color = theme[token.type as keyof typeof theme] || theme.text
        lineContent += token.content

        if (token.content.includes("\n")) {
          const lines = token.content.split("\n")
          lines.forEach((line, index) => {
            if (index > 0) {
              if (showLineNumbers) {
                const lineNumberSpan = document.createElement("span")
                lineNumberSpan.textContent = `${lineNumber} `
                lineNumberSpan.className = "line-number"
                lineNumberSpan.style.color = theme.lineNumber
                codeRef.current?.appendChild(lineNumberSpan)
              }
              lineNumber++
            }
            const lineSpan = document.createElement("span")
            lineSpan.textContent = line
            lineSpan.className = `token ${token.type}`
            lineSpan.style.color = theme[token.type as keyof typeof theme] || theme.text
            codeRef.current?.appendChild(lineSpan)
            if (index < lines.length - 1) {
              codeRef.current?.appendChild(document.createElement("br"))
            }
          })
        } else {
          codeRef.current?.appendChild(span)
        }

        if (lineContent.includes("\n")) {
          lineContent = ""
        }
      })

      // Add the last line number if it doesn't end with a newline
      if (showLineNumbers && !lineContent.endsWith("\n")) {
        const lineNumberSpan = document.createElement("span")
        lineNumberSpan.textContent = `${lineNumber} `
        lineNumberSpan.className = "line-number"
        lineNumberSpan.style.color = theme.lineNumber
        codeRef.current?.insertBefore(lineNumberSpan, codeRef.current.firstChild)
      }
    }
  }, [code, language, theme, showLineNumbers])

  const toggleExpand = () => {
    setIsExpanded(!isExpanded)
  }

  return (
    <div
      className={`syntax-highlighter ${isDarkMode ? "dark" : ""}`}
      style={{ maxHeight: isExpanded ? "none" : maxHeight, overflow: "auto" }}
    >
      <pre
        ref={codeRef}
        className={`language-${language} bg-white dark:bg-gray-900 p-4 rounded-md ${
          wrapLines ? "whitespace-pre-wrap" : "whitespace-pre"
        }`}
      />
      {expandable && (
        <button
          onClick={toggleExpand}
          className="expand-collapse-btn w-full flex items-center justify-center p-2 bg-gray-100 dark:bg-gray-800 text-gray-700 dark:text-gray-300 hover:bg-gray-200 dark:hover:bg-gray-700 transition-colors duration-200"
        >
          {isExpanded ? <ChevronUp className="mr-2" /> : <ChevronDown className="mr-2" />}
          {isExpanded ? "Collapse" : "Expand"}
        </button>
      )}
    </div>
  )
}

