import React from "react"
import { createContext, useState, useEffect } from "react"
import type { Theme, ThemeContextType } from "../types"
import { lightTheme, darkTheme } from "../themes"

export const ThemeContext = createContext<ThemeContextType>({
  theme: lightTheme,
  setTheme: () => {},
  isDarkMode: false,
  setIsDarkMode: () => {},
})

export const ThemeProvider: React.FC<{ children: React.ReactNode; initialIsDarkMode?: boolean }> = ({
  children,
  initialIsDarkMode = false,
}) => {
  const [theme, setTheme] = useState<Theme>(initialIsDarkMode ? darkTheme : lightTheme)
  const [isDarkMode, setIsDarkMode] = useState<boolean>(initialIsDarkMode)

  useEffect(() => {
    const root = window.document.documentElement
    const darkModeMediaQuery = window.matchMedia("(prefers-color-scheme: dark)")

    const updateTheme = () => {
      if (isDarkMode || (!isDarkMode && darkModeMediaQuery.matches)) {
        root.classList.add("dark")
        setTheme(darkTheme)
      } else {
        root.classList.remove("dark")
        setTheme(lightTheme)
      }
    }

    updateTheme()
    darkModeMediaQuery.addListener(updateTheme)

    return () => darkModeMediaQuery.removeListener(updateTheme)
  }, [isDarkMode])

  return (
    <ThemeContext.Provider value={{ theme, setTheme, isDarkMode, setIsDarkMode }}>{children}</ThemeContext.Provider>
  )
}

