import type { Token, TokenType } from "../types"

const keywords = new Set([
  "const",
  "let",
  "var",
  "function",
  "return",
  "if",
  "else",
  "for",
  "while",
  "do",
  "switch",
  "case",
  "default",
  "break",
  "continue",
  "class",
  "extends",
  "implements",
  "import",
  "export",
  "from",
  "as",
  "async",
  "await",
  "try",
  "catch",
  "finally",
  "throw",
  "typeof",
  "instanceof",
  "in",
  "of",
  "new",
  "this",
  "super",
  "static",
  "public",
  "private",
  "protected",
  "get",
  "set",
  "interface",
  "type",
  "enum",
  "namespace",
  "module",
  "declare",
  "abstract",
  "implements",
  "readonly",
])

const builtins = new Set([
  "console",
  "Math",
  "Date",
  "Array",
  "Object",
  "String",
  "Number",
  "Boolean",
  "RegExp",
  "Map",
  "Set",
  "Promise",
  "Symbol",
  "Error",
  "JSON",
  "Infinity",
  "NaN",
  "undefined",
  "null",
  "true",
  "false",
])

function isAlpha(char: string): boolean {
  return /[a-zA-Z_$]/.test(char)
}

function isAlphaNumeric(char: string): boolean {
  return /[a-zA-Z0-9_$]/.test(char)
}

function isDigit(char: string): boolean {
  return /[0-9]/.test(char)
}

export function tokenize(code: string | undefined, language: string): Token[] {
  if (!code) return []
  const tokens: Token[] = []
  let current = 0

  function addToken(type: TokenType, content: string) {
    tokens.push({ type, content })
  }

  while (current < code.length) {
    const char = code[current]

    // Handle comments
    if (char === "/" && code[current + 1] === "/") {
      let value = ""
      current += 2
      while (current < code.length && code[current] !== "\n") {
        value += code[current]
        current++
      }
      addToken("comment", "//" + value)
      continue
    }

    // Handle multi-line comments
    if (char === "/" && code[current + 1] === "*") {
      let value = ""
      current += 2
      while (current < code.length && !(code[current] === "*" && code[current + 1] === "/")) {
        value += code[current]
        current++
      }
      if (current < code.length) {
        value += "*/"
        current += 2
      }
      addToken("comment", "/*" + value)
      continue
    }

    // Handle strings
    if (char === '"' || char === "'" || char === "`") {
      let value = char
      const openQuote = char
      current++
      while (current < code.length && code[current] !== openQuote) {
        if (code[current] === "\\" && current + 1 < code.length) {
          value += code[current] + code[current + 1]
          current += 2
        } else {
          value += code[current]
          current++
        }
      }
      if (current < code.length) {
        value += openQuote
        current++
      }
      addToken("string", value)
      continue
    }

    // Handle numbers
    if (isDigit(char) || (char === "." && isDigit(code[current + 1]))) {
      let value = ""
      let isFloat = false
      while (current < code.length && (isDigit(code[current]) || code[current] === ".")) {
        if (code[current] === ".") {
          if (isFloat) break
          isFloat = true
        }
        value += code[current]
        current++
      }
      // Handle exponents
      if (current < code.length && (code[current] === "e" || code[current] === "E")) {
        value += code[current]
        current++
        if (current < code.length && (code[current] === "+" || code[current] === "-")) {
          value += code[current]
          current++
        }
        while (current < code.length && isDigit(code[current])) {
          value += code[current]
          current++
        }
      }
      addToken("number", value)
      continue
    }

    // Handle identifiers, keywords, and builtins
    if (isAlpha(char)) {
      let value = ""
      while (current < code.length && isAlphaNumeric(code[current])) {
        value += code[current]
        current++
      }
      if (keywords.has(value)) {
        addToken("keyword", value)
      } else if (builtins.has(value)) {
        addToken("builtin", value)
      } else {
        // Check if it's a function or class declaration
        let nextNonSpace = current
        while (nextNonSpace < code.length && /\s/.test(code[nextNonSpace])) {
          nextNonSpace++
        }
        if (code[nextNonSpace] === "(") {
          addToken("function", value)
        } else if (value[0] === value[0].toUpperCase()) {
          addToken("class", value)
        } else {
          addToken("identifier", value)
        }
      }
      continue
    }

    // Handle operators
    if (/[+\-*/%=<>!&|^~?:]/.test(char)) {
      let value = char
      current++
      while (current < code.length && /[+\-*/%=<>!&|^~?:]/.test(code[current])) {
        value += code[current]
        current++
      }
      addToken("operator", value)
      continue
    }

    // Handle punctuation
    if (/[(){}[\].,;]/.test(char)) {
      addToken("punctuation", char)
      current++
      continue
    }

    // Handle decorators (for TypeScript)
    if (char === "@" && isAlpha(code[current + 1])) {
      let value = "@"
      current++
      while (current < code.length && isAlphaNumeric(code[current])) {
        value += code[current]
        current++
      }
      addToken("decorator", value)
      continue
    }

    // Handle regular expressions
    if (char === "/" && tokens.length > 0 && tokens[tokens.length - 1].type !== "identifier") {
      let value = "/"
      let inCharSet = false
      current++
      while (current < code.length) {
        if (code[current] === "\\") {
          value += code[current] + code[current + 1]
          current += 2
        } else if (code[current] === "[") {
          inCharSet = true
          value += code[current]
          current++
        } else if (code[current] === "]" && inCharSet) {
          inCharSet = false
          value += code[current]
          current++
        } else if (code[current] === "/" && !inCharSet) {
          value += code[current]
          current++
          // Handle regex flags
          while (current < code.length && isAlpha(code[current])) {
            value += code[current]
            current++
          }
          addToken("regex", value)
          break
        } else {
          value += code[current]
          current++
        }
      }
      if (value !== "/") {
        continue
      }
    }

    // Handle whitespace
    if (/\s/.test(char)) {
      let value = ""
      while (current < code.length && /\s/.test(code[current])) {
        value += code[current]
        current++
      }
      addToken("whitespace", value)
      continue
    }

    // If we've gotten this far, we don't know what it is
    addToken("unknown", char)
    current++
  }

  return tokens
}

